//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

namespace DesignSurfaceExtenderLibrary.Extensions
{
    internal class AnchorGlyph : ExtensionGlyph
    {
        // This defines the bounds used for hit testing.
        // These bounds are typically different than the bounds 
        // of the glyph itself.
        protected Rectangle hitBoundsValue;

        // This is the cursor returned if hit test is positive.
        protected Cursor hitTestCursor = Cursors.Hand;

        // This defines the AnchorStyle which this glyph represents.
        private AnchorStyles anchorStyleValue;

        // This defines the size of the anchor glyphs.
        private const int glyphSize = 6;

        // This defines the size of the hit bounds for an AnchorGlyph.
        private const int hitBoundSize = glyphSize + 4;

        internal AnchorGlyph(
            Control c,
            AnchorStyles anchorStyle,
            BehaviorService behaviorService,
            IComponentChangeService changeService,
            ISelectionService selectionService,
            IReferenceService referenceService,
            IServiceProvider serviceProvider,
            Adorner adorner )
            : base(
                c,
                behaviorService,
                changeService,
                selectionService,
                referenceService,
                serviceProvider,
                new AnchorBehavior(),
                adorner 
             )
        {   
            this.anchorStyleValue = anchorStyle;

            // Compute the bounds of this glyph.
            this.ComputeGeometry();
        }

        internal AnchorStyles AnchorStyle
        {
            get
            {
                return this.anchorStyleValue;
            }
        }

        ///////////////////////////////////////////////////////////////////////
        #region Overrides

        // This method renders the AnchorGlyph as a filled rectangle
        // if the glyph is enabled, or as an open rectangle if the
        // glyph is disabled.
        public override void Paint( PaintEventArgs e )
        {
            if( this.IsEnabled )
            {
                using( Brush b = new SolidBrush( Color.Tomato ) )
                {
                    e.Graphics.FillRectangle( b, this.Bounds );
                }
            }
            else
            {
                using( Pen p = new Pen( Color.Tomato ) )
                {
                    e.Graphics.DrawRectangle( p, this.Bounds );
                }
            }
        }

        // An AnchorGlyph has keyboard and mouse interaction, so it's
        // important to return a cursor when the mouse is located in 
        // the glyph's hit region. When this occurs, the 
        // AnchorBehavior becomes active.

        public override Cursor GetHitTest(Point p)
        {
            if (hitBoundsValue.Contains(p))
            {
                return hitTestCursor;
            }

            return null;
        }

        #endregion

        protected override ReadOnlyCollection<string> PopulateDependentProperties()
        {
            List<string> properties = new List<string>();

            properties.Add( "Anchor" );
            properties.Add( "Size" );
            properties.Add( "Height" );
            properties.Add( "Width" );
            properties.Add( "Location" );

            return new ReadOnlyCollection<string>( properties );
        }

        #region Implementation

        // This utility method computes the position and size of 
        // the AnchorGlyph in the Adorner window's coordinates.
        // It also computes the hit test bounds, which are
        // slightly larger than the glyph's bounds.
        protected override void ComputeGeometry()
        {
            Rectangle translatedBounds = new Rectangle(
                this.behaviorService.ControlToAdornerWindow( base.RelatedControl ),
                base.RelatedControl.Size);

            if ((this.anchorStyleValue & AnchorStyles.Top) == AnchorStyles.Top)
            {
                this.boundsValue = new Rectangle(
                    translatedBounds.X + (translatedBounds.Width / 2) - (glyphSize / 2),
                    translatedBounds.Y + glyphSize,
                    glyphSize,
                    glyphSize);
            }
            if ((this.anchorStyleValue & AnchorStyles.Bottom) == AnchorStyles.Bottom)
            {
                this.boundsValue = new Rectangle(
                    translatedBounds.X + (translatedBounds.Width / 2) - (glyphSize / 2),
                    translatedBounds.Bottom - 2 * glyphSize,
                    glyphSize,
                    glyphSize);
            }
            if ((this.anchorStyleValue & AnchorStyles.Left) == AnchorStyles.Left)
            {
                this.boundsValue = new Rectangle(
                    translatedBounds.X + glyphSize,
                    translatedBounds.Y + (translatedBounds.Height / 2) - (glyphSize / 2),
                    glyphSize,
                    glyphSize);
            }
            if ((this.anchorStyleValue & AnchorStyles.Right) == AnchorStyles.Right)
            {
                this.boundsValue = new Rectangle(
                    translatedBounds.Right - 2 * glyphSize,
                    translatedBounds.Y + (translatedBounds.Height / 2) - (glyphSize / 2),
                    glyphSize,
                    glyphSize);
            }

            this.hitBoundsValue = new Rectangle(
                this.boundsValue.Left - hitBoundSize / 2,
                this.boundsValue.Top - hitBoundSize / 2,
                hitBoundSize,
                hitBoundSize);
        }

        // This utility property determines if the AnchorGlyph is 
        // enabled, according to the value specified by the 
        // control's Anchor property.
        internal bool IsEnabled
        {
            get
            {
                return ( ( this.anchorStyleValue & base.RelatedControl.Anchor ) == this.anchorStyleValue );
            }
        }

        #endregion 
    }
}
